function [GDiskMode, TempImageCache] = voxelcolor_reconstruct(dataset, vcs, parameters, GDiskMode, TempImageCache)
% This function performs the actual 3D reconstruction by first loading the
% input images and their camera calibration from disk (or from cache) and
% then performing the actual reconstruction.
% More documentation in future versions, I have to go on holiday now ;)

[pathstr,name,ext] = fileparts(dataset);
a = textread(dataset, '%s');
GDataPath = [pathstr '/' char(a(1))];
GDataPrefix = char(a(2));
GDataStart = str2num(char(a(3)));

disp(['Loading dataset from ' GDataPath ' with prefix ' GDataPrefix ' and starting at number ' int2str(GDataStart)]);

if GDiskMode < 2
    [images, w, h, cameras, imageList, cameraList, rawFileName] = voxelcolor_load_sequence( ...
                                                                  GDataPath, ...
                                                                  GDataPrefix, ...
                                                                  GDataStart, ...
                                                                  'cpcc', ...
                                                                  GDiskMode, 1);

    if length(TempImageCache) == 0
        TempImageCache = 'cache.mat';
    end
    
    % clear existing cache if it exists
    voxelcolor_clearcache(TempImageCache);

    datasetCopy = dataset;
    save(TempImageCache, 'images', 'w', 'h', 'cameras', 'imageList', 'cameraList', 'rawFileName', 'datasetCopy');
    if GDiskMode == 1
        GDiskMode = 2;           % we have now stored it, next time just load
    end
else
    disp(['Using cached images from ' TempImageCache]);
    load(TempImageCache)         % use dumped images in 'images.raw' and images, w, h, cameras from 'data.mat'
    if strcmp(dataset, datasetCopy) == 0
        disp('Cache does not match storage: different dataset name. Try again, cache was cleared.');
        voxelcolor_clearcache(TempImageCache);
        GDiskMode = 1;
        return
    end
end

clear voxels
disp('Starting voxel coloring...');
if length(rawFileName) > 0
    % images are transmitted through disk - pass the filename of the disk file
    voxels = voxelcolor(images, cameras, parameters, vcs, rawFileName);
else
    voxels = voxelcolor(images, cameras, parameters, vcs);
end
disp('Voxel coloring done.');

% post-processing in MatLab
% shows number of voxels per layer and a side view from one side
% (so you can quickly see how 'good' your reconstruction is)
clear pic
clear res
clear t
clear m

t = voxels(:, :, :, 2:4);
siz = size(t,1);
res = zeros(size(t,2), size(t,3), 3);

for i = siz:-1:1
  pic(:, :, :) = t(i, :, :, :);
  m(:, :, 1) = voxels(i, :, :, 1);
  m(:, :, 2) = voxels(i, :, :, 1);
  m(:, :, 3) = voxels(i, :, :, 1);
  res = res .* (~m);
  res = res + double(pic);
end
  
    j = max(max(max(res)));
    if j == 0 
        j = 1;
    end
    figure;
imshow(double(res) ./ double(j))

for i = 1:size(t,3)
  k(i) = sum(sum(sum(voxels(:, :, i,1))));
end
disp(['Number of voxels in the reconstruction: ' int2str(sum(k))]);

disp('Saving reconstruction to disk...');
voxelcolor_save_volume('reconstruction.voxelraw', voxels);

if GDiskMode == 0
    voxelcolor_clearcache(TempImageCache);
end
